@@ -13,7 +13,8 @@ import platform
 import uuid
 import os
 import json
-from typing import Dict, Any, List, Optional
+from pathlib import Path
+from typing import Dict, Any, List, Optional, Callable, Iterator
 
 from .auth import Auth
 from .config import Config
@@ -33,14 +34,31 @@ class APIError(Exception):
 
 
 class APIClient:
-    """
-    Client for interacting with HLA-Compass API
+    """Client for interacting with the HLA-Compass API.
+
+    Highlights
+    ---------
+    • **Iterators** – Use :meth:`iter_peptides`, :meth:`iter_proteins`, etc. to stream
+      large result sets without manual paging::
+
+          client = APIClient()
+          for peptide in client.iter_peptides({"sequence": "SIINFEKL"}):
+              print(peptide["sequence"], peptide["mass"])
+
+    • **Rate limits** – Override defaults via environment variables
+      ``HLA_RATE_LIMIT_MAX_REQUESTS`` and ``HLA_RATE_LIMIT_TIME_WINDOW``; inspect the
+      active settings with :meth:`get_rate_limit_state`.
+
+    • **Telemetry** – Attach a callback to capture structured metrics for every
+      request (status, duration, retries)::
+
+          def log_request(data: dict[str, Any]) -> None:
+              print("API", data["status_code"], data["duration_seconds"], data["url"])
+
+          client = APIClient()
+          client.set_telemetry_callback(log_request)
+          client.get_peptide("pep_123")
 
-    This client handles:
-    - Authentication using JWT tokens
-    - Request formatting and response parsing
-    - Error handling and retries
-    - Pagination support
     """
 
     def __init__(self, provider: str = None, catalog: str = None):
@@ -58,10 +76,20 @@ class APIClient:
         self.provider = provider or self.config.get("data_provider", "alithea-bio")
         self.catalog = catalog or self.config.get("data_catalog", "immunopeptidomics")
         
+        rl_settings = Config.get_rate_limit_settings()
+        max_requests = rl_settings.get("max_requests", 100)
+        time_window = rl_settings.get("time_window", 60)
+
         self.rate_limiter = RateLimiter(
-            max_requests=100,  # 100 requests per minute
-            time_window=60,
+            max_requests=max_requests,
+            time_window=time_window,
         )
+        self._rate_limit_state: Dict[str, Any] = {
+            "max_requests": max_requests,
+            "time_window": time_window,
+            "last_wait_seconds": 0.0,
+        }
+        self._telemetry_callback: Optional[Callable[[Dict[str, Any]], None]] = None
 
         # Create a persistent HTTP session for connection reuse and default headers
         self.session = requests.Session()
@@ -170,6 +198,7 @@ class APIClient:
 
         # Apply rate limiting
         self.rate_limiter.acquire()
+        self._rate_limit_state["last_wait_seconds"] = self.rate_limiter.last_wait
 
         # Get combined headers (session defaults + auth)
         headers = self._headers()
@@ -185,9 +214,11 @@ class APIClient:
 
         for attempt in range(max_retries):
             try:
+                start_time = time.perf_counter()
                 # Create a fresh set of headers per attempt and add a unique request id
                 attempt_headers = dict(headers)
-                attempt_headers["X-Request-Id"] = str(uuid.uuid4())
+                request_id = str(uuid.uuid4())
+                attempt_headers["X-Request-Id"] = request_id
                 # Add idempotency key for POSTs to make retries safe
                 if method_upper == "POST" and idempotency_key:
                     attempt_headers.setdefault("Idempotency-Key", idempotency_key)
@@ -200,6 +231,7 @@ class APIClient:
                     json=json_data,
                     timeout=(5, 30),
                 )
+                duration = time.perf_counter() - start_time
 
                 # Handle 401 - try to refresh token once
                 if response.status_code == 401 and attempt == 0:
@@ -248,18 +280,77 @@ class APIClient:
                 if response.status_code >= 400:
                     # Sanitize error message to avoid info disclosure
                     error_msg = parse_api_error(response, "API request failed")
+                    self._emit_telemetry(
+                        method=method_upper,
+                        url=url,
+                        status_code=response.status_code,
+                        duration=duration,
+                        attempt=attempt,
+                        retries=max_retries,
+                        params=params,
+                        idempotency_key=idempotency_key,
+                        request_id=request_id,
+                        error=error_msg,
+                    )
                     raise APIError(error_msg, response.status_code)
 
-                # Parse successful response
-                data = response.json()
+                # Parse successful response (204 = empty body)
+                if response.status_code == 204:
+                    data = {}
+                else:
+                    try:
+                        data = response.json()
+                    except AttributeError:
+                        # Some test doubles may not expose .json(); treat as empty
+                        data = {}
+                    except ValueError:
+                        body = getattr(response, 'text', None)
+                        if body in (None, ''):
+                            body = getattr(response, 'content', b'')
+                        if not body:
+                            data = {}
+                        else:
+                            error_msg = parse_api_error(
+                                response, "Invalid JSON response from API"
+                            )
+                            raise APIError(error_msg, response.status_code)
 
                 # Handle success wrapper format
-                if isinstance(data, dict) and data.get("success"):
-                    return data.get("data", data)
+                payload = (
+                    data.get("data", data)
+                    if isinstance(data, dict) and data.get("success")
+                    else data
+                )
 
-                return data
+                self._emit_telemetry(
+                    method=method_upper,
+                    url=url,
+                    status_code=response.status_code,
+                    duration=duration,
+                    attempt=attempt,
+                    retries=max_retries,
+                    params=params,
+                    idempotency_key=idempotency_key,
+                    request_id=request_id,
+                )
+
+                return payload
+
+            except requests.Timeout as e:
+                duration = time.perf_counter() - start_time
+                self._emit_telemetry(
+                    method=method_upper,
+                    url=url,
+                    status_code=None,
+                    duration=duration,
+                    attempt=attempt,
+                    retries=max_retries,
+                    params=params,
+                    idempotency_key=idempotency_key,
+                    request_id=request_id,
+                    error="timeout",
+                )
 
-            except requests.Timeout:
                 if attempt < max_retries - 1:
                     wait_time = 2**attempt
                     logger.warning(f"Request timeout, retrying in {wait_time} seconds")
@@ -268,9 +359,23 @@ class APIClient:
                 else:
                     raise APIError(
                         "Request timed out. Please check your connection and try again."
-                    )
+                    ) from e
 
             except requests.RequestException as e:
+                duration = time.perf_counter() - start_time
+                self._emit_telemetry(
+                    method=method_upper,
+                    url=url,
+                    status_code=None,
+                    duration=duration,
+                    attempt=attempt,
+                    retries=max_retries,
+                    params=params,
+                    idempotency_key=idempotency_key,
+                    request_id=request_id,
+                    error=str(e),
+                )
+
                 if attempt < max_retries - 1 and "connection" in str(e).lower():
                     wait_time = 2**attempt
                     logger.warning(f"Connection error, retrying in {wait_time} seconds")
@@ -279,6 +384,39 @@ class APIClient:
                 else:
                     raise APIError(f"Network error: {str(e)}")
 
+    def _iterate_paginated(
+        self,
+        fetcher: Callable[[int, int], List[Dict[str, Any]]],
+        page_size: int,
+        max_results: Optional[int],
+    ) -> Iterator[Dict[str, Any]]:
+        offset = 0
+        yielded = 0
+
+        while True:
+            if max_results is not None:
+                remaining = max_results - yielded
+                if remaining <= 0:
+                    break
+                limit = min(page_size, remaining)
+            else:
+                limit = page_size
+
+            page = fetcher(limit, offset)
+            if not page:
+                break
+
+            for item in page:
+                yield item
+                yielded += 1
+                if max_results is not None and yielded >= max_results:
+                    return
+
+            if len(page) < limit:
+                break
+
+            offset += limit
+
     # Peptide endpoints
 
     def get_peptides(
@@ -331,6 +469,19 @@ class APIClient:
         result = self._make_request("GET", self._build_data_endpoint("peptides", peptide_id))
         return result.get("peptide", result)
 
+    def iter_peptides(
+        self,
+        filters: Dict = None,
+        page_size: int = 100,
+        max_results: Optional[int] = None,
+    ) -> Iterator[Dict[str, Any]]:
+        """Yield peptides across pages without manual pagination."""
+
+        def fetch(limit: int, offset: int) -> List[Dict[str, Any]]:
+            return self.get_peptides(filters=filters, limit=limit, offset=offset)
+
+        yield from self._iterate_paginated(fetch, page_size, max_results)
+
     def get_peptide_samples(self, peptide_id: str) -> List[Dict[str, Any]]:
         """Get samples containing a peptide"""
         result = self._make_request(
@@ -388,6 +539,19 @@ class APIClient:
         result = self._make_request("GET", self._build_data_endpoint("proteins", protein_id))
         return result.get("protein", result)
 
+    def iter_proteins(
+        self,
+        filters: Dict = None,
+        page_size: int = 100,
+        max_results: Optional[int] = None,
+    ) -> Iterator[Dict[str, Any]]:
+        """Yield proteins across pages without manual pagination."""
+
+        def fetch(limit: int, offset: int) -> List[Dict[str, Any]]:
+            return self.get_proteins(filters=filters, limit=limit, offset=offset)
+
+        yield from self._iterate_paginated(fetch, page_size, max_results)
+
     def get_protein_peptides(self, protein_id: str) -> List[Dict[str, Any]]:
         """Get peptides from a protein"""
         result = self._make_request(
@@ -435,6 +599,19 @@ class APIClient:
         result = self._make_request("GET", self._build_data_endpoint("samples", sample_id))
         return result.get("sample", result)
 
+    def iter_samples(
+        self,
+        filters: Dict = None,
+        page_size: int = 100,
+        max_results: Optional[int] = None,
+    ) -> Iterator[Dict[str, Any]]:
+        """Yield samples across pages without manual pagination."""
+
+        def fetch(limit: int, offset: int) -> List[Dict[str, Any]]:
+            return self.get_samples(filters=filters, limit=limit, offset=offset)
+
+        yield from self._iterate_paginated(fetch, page_size, max_results)
+
     def get_sample_peptides(self, sample_id: str) -> List[Dict[str, Any]]:
         """Get peptides from a sample"""
         result = self._make_request(
@@ -499,11 +676,32 @@ class APIClient:
         result = self._make_request("GET", "/v1/modules", params=params)
         return result.get("modules", result.get("items", []))
 
+    def iter_modules(
+        self,
+        category: str = None,
+        page_size: int = 100,
+        max_results: Optional[int] = None,
+    ) -> Iterator[Dict[str, Any]]:
+        """Yield modules across pages without manual pagination."""
+
+        def fetch(limit: int, offset: int) -> List[Dict[str, Any]]:
+            params = {"limit": limit}
+            if category:
+                params["category"] = category
+            params["offset"] = offset
+            result = self._make_request("GET", "/v1/modules", params=params)
+            if isinstance(result, dict):
+                return result.get("modules", result.get("items", []))
+            return result if isinstance(result, list) else []
+
+        yield from self._iterate_paginated(fetch, page_size, max_results)
+
     def upload_module(
         self, module_path: str, module_name: str, version: str
     ) -> Dict[str, Any]:
         """
-        Upload module package to the real API endpoint using multipart/form-data.
+        Upload module package to the real API endpoint using JSON (base64) payloads,
+        with a multipart/form-data fallback for legacy environments.
 
         Args:
             module_path: Path to module zip file
@@ -527,39 +725,51 @@ class APIClient:
         metadata = {
             "name": module_name,
             "version": version,
+            "filename": os.path.basename(module_path),
+            "size": file_size,
         }
-        # Base headers + Accept; Content-Type will be set by requests for multipart
-        headers = self._headers()
-        headers.setdefault("Accept", "application/json")
-        # Idempotency for POST
-        idem = str(uuid.uuid4())
-        headers.setdefault("Idempotency-Key", idem)
 
-        # Two attempts: try, then refresh on 401 once
+        base_headers = self._headers()
+        base_headers.setdefault("Accept", "application/json")
+        idem = str(uuid.uuid4())
+        base_headers.setdefault("Idempotency-Key", idem)
+
         for attempt in range(2):
-            with open(module_path, "rb") as f:
-                files = {
-                    "module": (os.path.basename(module_path), f, "application/zip"),
-                }
-                data = {
-                    "metadata": json.dumps(metadata),
-                }
-                resp = self.session.post(
-                    url,
-                    headers=headers,
-                    files=files,
-                    data=data,
-                    timeout=(5, 60),
-                )
+            headers = dict(base_headers)
+            headers.pop("Content-Type", None)
+
+            try:
+                with open(module_path, "rb") as fh:
+                    fh.seek(0)
+                    files = {
+                        "module": (
+                            os.path.basename(module_path),
+                            fh,
+                            "application/zip",
+                        )
+                    }
+                    form_data = {"metadata": json.dumps(metadata)} if metadata else None
+
+                    resp = self.session.post(
+                        url,
+                        headers=headers,
+                        files=files,
+                        data=form_data,
+                        timeout=(5, 120),
+                    )
+            except OSError as exc:
+                raise APIError(f"Failed to read module package: {exc}")
 
             if resp.status_code == 401 and attempt == 0:
-                # try token refresh then retry once
                 if self.auth.refresh_token():
-                    headers = self._headers()
-                    headers.setdefault("Accept", "application/json")
-                    headers.setdefault("Idempotency-Key", idem)
+                    base_headers = self._headers()
+                    base_headers.setdefault("Accept", "application/json")
+                    base_headers.setdefault("Idempotency-Key", idem)
                     continue
-                raise APIError("Authentication expired. Please run 'hla-compass auth login' again", 401)
+                raise APIError(
+                    "Authentication expired. Please run 'hla-compass auth login' again",
+                    401,
+                )
 
             if resp.status_code >= 400:
                 # best-effort parse
@@ -567,7 +777,11 @@ class APIClient:
                     err = resp.json()
                 except Exception:
                     err = {"message": resp.text}
-                raise APIError(parse_api_error(resp, "Upload failed"), resp.status_code, details=err)
+                raise APIError(
+                    parse_api_error(resp, "Upload failed"),
+                    resp.status_code,
+                    details=err,
+                )
 
             try:
                 payload = resp.json()
@@ -579,14 +793,56 @@ class APIClient:
             if not module_id:
                 raise APIError("Upload succeeded but module_id not returned")
 
-            # Normalize return for CLI
             normalized = {"module_id": module_id}
             normalized.update(data)
             return normalized
 
-        # Should not reach
         raise APIError("Upload failed after token refresh attempt")
 
+    def get_rate_limit_state(self) -> Dict[str, Any]:
+        """Return the active rate limit configuration and last wait duration."""
+        return dict(self._rate_limit_state)
+
+    def _emit_telemetry(
+        self,
+        *,
+        method: str,
+        url: str,
+        status_code: Optional[int],
+        duration: float,
+        attempt: int,
+        retries: int,
+        params: Optional[Dict[str, Any]],
+        idempotency_key: Optional[str],
+        request_id: str,
+        error: Optional[str] = None,
+    ) -> None:
+        payload = {
+            "method": method,
+            "url": url,
+            "status_code": status_code,
+            "duration_seconds": round(duration, 4),
+            "attempt": attempt + 1,
+            "max_retries": retries,
+            "rate_limit_wait": self._rate_limit_state.get("last_wait_seconds"),
+            "params": params or {},
+            "idempotency_key": idempotency_key,
+            "request_id": request_id,
+            "error": error,
+        }
+
+        if self._telemetry_callback:
+            try:
+                self._telemetry_callback(dict(payload))
+            except Exception as exc:  # pragma: no cover - user callback
+                logger.warning("Telemetry callback raised an exception: %s", exc)
+
+        log_method = logger.info
+        if error or (status_code is not None and status_code >= 400):
+            log_method = logger.warning
+
+        log_method("API request telemetry", extra={"api_request": payload})
+
     def register_module(
         self, module_id: str, metadata: Dict[str, Any]
     ) -> Dict[str, Any]:
@@ -605,3 +861,14 @@ class APIClient:
             "notes": metadata.get("description", "Module published via SDK"),
         }
         return self._make_request("PUT", f"/v1/modules/{module_id}/publish", json_data=json_data)
+
+    def register_container_module(self, payload: Dict[str, Any]) -> Dict[str, Any]:
+        """Register a container-based module release with the control plane."""
+
+        return self._make_request("POST", "/v1/modules/container", json_data=payload)
+
+    def set_telemetry_callback(
+        self, callback: Optional[Callable[[Dict[str, Any]], None]]
+    ) -> None:
+        """Register a callback that receives request telemetry dictionaries."""
+        self._telemetry_callback = callback
