@@ -235,9 +235,17 @@ class PeptideData(BaseData):
                 )
 
             # Fall back to API if available
-            if self.api and hasattr(self.api, "search_peptides_by_hla"):
-                return self.api.search_peptides_by_hla(
-                    allele=allele, min_score=binding_score_min, limit=limit
+            if self.api:
+                # Prefer an explicit API method if provided
+                if hasattr(self.api, "search_peptides_by_hla"):
+                    return self.api.search_peptides_by_hla(
+                        allele=allele, min_score=binding_score_min, limit=limit
+                    )
+                # Generic fallback using get_peptides with mapped filters
+                return self.api.get_peptides(
+                    filters={"hla_allele": allele, "min_intensity": binding_score_min},
+                    limit=limit,
+                    offset=0,
                 )
 
             # If neither is available, raise error
