@@ -20,19 +20,14 @@ import asyncio
 import logging
 
 from watchdog.observers import Observer
-from watchdog.events import FileSystemEventHandler, FileModifiedEvent
+from watchdog.events import FileSystemEventHandler
 from aiohttp import web
-import aiohttp
 from aiohttp import ClientSession, ClientTimeout, WSMsgType, TCPConnector
 import ssl
 
 from rich.console import Console
-from rich.live import Live
 from rich.table import Table
 from rich.panel import Panel
-from rich.layout import Layout
-from rich.syntax import Syntax
-from rich.text import Text
 
 from .testing import ModuleTester
 from .module import Module
@@ -91,6 +86,7 @@ class ModuleDevServer:
         start_frontend: bool = False,
         frontend_port: int = 3000,
         ca_bundle: str | None = None,
+        verbose: bool = False,
     ):
         self.module_dir = Path(module_dir).resolve()
         self.port = port
@@ -110,6 +106,7 @@ class ModuleDevServer:
         self.frontend_proxy = frontend_proxy
         self.start_frontend_process = start_frontend
         self.frontend_port = frontend_port
+        self.verbose = verbose
 
         # TLS verification context for upstream API (online mode)
         self.ca_bundle = ca_bundle
@@ -154,6 +151,7 @@ class ModuleDevServer:
         
         # Frontend process (for UI modules)
         self.frontend_process = None
+        self.frontend_log_thread = None
         
     def _load_manifest(self) -> dict:
         """Load module manifest"""
@@ -254,13 +252,16 @@ class ModuleDevServer:
         self.app.router.add_get("/dev/data/list", self.handle_dev_data_list)
         self.app.router.add_get("/dev/data/file", self.handle_dev_data_file)
 
-        # Root dev UI
+        # Root route
         self.app.router.add_get("/", self.handle_index)
+        # Backend testing UI explicitly under /backend
+        self.app.router.add_get("/backend", self.handle_backend)
 
         # Optional frontend proxy under /ui
         if self.frontend_proxy:
-            # Proxy /ui root and any nested paths under /ui to the frontend dev server
-            self.app.router.add_get("/ui", self.handle_ui_proxy)
+            # Serve a lightweight wrapper at /ui that bootstraps the UI using the dev bundle
+            self.app.router.add_get("/ui", self.handle_ui_wrapper)
+            # Proxy any nested assets or HMR websocket paths under /ui if needed
             self.app.router.add_route("GET", "/ui/{tail:.*}", self.handle_ui_proxy)
 
         # Catch-all for /api/* to proxy selected routes to real API (online mode)
@@ -444,6 +445,102 @@ class ModuleDevServer:
         ctype = mimetypes.guess_type(str(target))[0] or "application/octet-stream"
         return web.FileResponse(path=target, headers={"Content-Type": ctype})
 
+    async def handle_ui_wrapper(self, request):
+        """Serve a wrapper HTML that loads the dev bundle and mounts the UI.
+        This avoids relying on webpack-dev-server to provide index.html and ensures
+        consistent bootstrapping. The dev bundle is expected to expose a UMD global
+        named 'ModuleUI' as configured in webpack (library: 'ModuleUI').
+        """
+        html = f'''<!DOCTYPE html>
+<html lang="en">
+<head>
+  <meta charset="utf-8" />
+  <meta name="viewport" content="width=device-width, initial-scale=1" />
+  <title>{self.module_name} - HLA-Compass UI Dev</title>
+  <link rel="preconnect" href="https://fonts.googleapis.com" />
+  <link rel="preconnect" href="https://fonts.gstatic.com" crossorigin />
+  <link href="https://fonts.googleapis.com/css2?family=Inter:wght@100..900&display=swap" rel="stylesheet" />
+  <!-- Ant Design base reset (optional) -->
+  <link rel="stylesheet" href="https://unpkg.com/antd@5/dist/reset.css" />
+  <style>
+    html, body, #root {{ height: 100%; margin: 0; }}
+    body {{ font-family: Inter, system-ui, -apple-system, Segoe UI, Roboto, Helvetica Neue, Arial, sans-serif; }}
+  </style>
+</head>
+<body>
+  <div id="root"></div>
+  <!-- Externals required by the dev UMD bundle (React, ReactDOM, Day.js, AntD, Icons) -->
+  <script crossorigin src="https://unpkg.com/react@18/umd/react.development.js"></script>
+  <script crossorigin src="https://unpkg.com/react-dom@18/umd/react-dom.development.js"></script>
+  <!-- Day.js core and plugins required by AntD v5 date components -->
+  <script src="https://unpkg.com/dayjs@1/dayjs.min.js"></script>
+  <script src="https://unpkg.com/dayjs@1/plugin/customParseFormat.js"></script>
+  <script src="https://unpkg.com/dayjs@1/plugin/advancedFormat.js"></script>
+  <script src="https://unpkg.com/dayjs@1/plugin/weekday.js"></script>
+  <script src="https://unpkg.com/dayjs@1/plugin/localeData.js"></script>
+  <script src="https://unpkg.com/dayjs@1/plugin/weekOfYear.js"></script>
+  <script src="https://unpkg.com/dayjs@1/plugin/weekYear.js"></script>
+  <script>
+    // Extend dayjs with commonly used plugins before AntD initializes
+    (function() {{
+      try {{
+        if (window.dayjs) {{
+          if (window.dayjs_plugin_customParseFormat) dayjs.extend(window.dayjs_plugin_customParseFormat);
+          if (window.dayjs_plugin_advancedFormat) dayjs.extend(window.dayjs_plugin_advancedFormat);
+          if (window.dayjs_plugin_weekday) dayjs.extend(window.dayjs_plugin_weekday);
+          if (window.dayjs_plugin_localeData) dayjs.extend(window.dayjs_plugin_localeData);
+          if (window.dayjs_plugin_weekOfYear) dayjs.extend(window.dayjs_plugin_weekOfYear);
+          if (window.dayjs_plugin_weekYear) dayjs.extend(window.dayjs_plugin_weekYear);
+        }} else {{
+          console.error('dayjs failed to load before AntD.');
+        }}
+      }} catch (e) {{
+        console.error('Failed to extend dayjs plugins:', e);
+      }}
+    }})();
+  </script>
+  <script src="https://unpkg.com/antd@5/dist/antd.min.js"></script>
+  <script src="https://unpkg.com/@ant-design/icons@6/dist/index.umd.js"></script>
+  <!-- Dev bundle from webpack-dev-server -->
+  <script src="http://localhost:{self.frontend_port}/bundle.js"></script>
+  <script>
+    (function() {{
+      // Helper to call module execute via dev server API
+      async function onExecute(params) {{
+        const res = await fetch('/api/execute', {{
+          method: 'POST', headers: {{ 'Content-Type': 'application/json' }},
+          body: JSON.stringify({{ input: params }})
+        }});
+        return res.json();
+      }}
+      function mount() {{
+        const rootEl = document.getElementById('root');
+        const Comp = (window.ModuleUI && (window.ModuleUI.default || window.ModuleUI)) || null;
+        if (!Comp) {{
+          rootEl.innerHTML = '<pre style="padding:16px;color:#b91c1c">Error: ModuleUI UMD not found. Ensure webpack output.library is set to \"ModuleUI\".</pre>';
+          return;
+        }}
+        const el = React.createElement(Comp, {{ onExecute }});
+        if (window.ReactDOM && window.ReactDOM.createRoot) {{
+          window.ReactDOM.createRoot(rootEl).render(el);
+        }} else if (window.ReactDOM && window.ReactDOM.render) {{
+          window.ReactDOM.render(el, rootEl);
+        }} else {{
+          rootEl.innerHTML = '<pre style="padding:16px;color:#b91c1c">Error: ReactDOM not available.</pre>';
+        }}
+      }}
+      // Try mounting after a short delay to ensure bundle is loaded
+      if (document.readyState === 'complete' || document.readyState === 'interactive') {{
+        setTimeout(mount, 0);
+      }} else {{
+        window.addEventListener('DOMContentLoaded', mount);
+      }}
+    }})();
+  </script>
+</body>
+</html>'''
+        return web.Response(text=html, content_type="text/html")
+
     async def handle_ui_proxy(self, request):
         """Proxy /ui (and subpaths) to the local frontend dev server."""
         # Map /ui and /ui/* to upstream path
@@ -519,7 +616,11 @@ class ModuleDevServer:
             )
 
     async def handle_index(self, request):
-        """Serve the development UI"""
+        """Serve the development UI or redirect to UI for with-ui modules"""
+        # If this is a UI module and frontend proxy is enabled, send users to the UI by default
+        if self.module_type == "with-ui" and self.frontend_proxy:
+            raise web.HTTPFound(location="/ui")
+        # Otherwise, show the backend testing UI
         html = self._generate_dev_ui()
         return web.Response(text=html, content_type="text/html")
     
@@ -537,6 +638,11 @@ class ModuleDevServer:
             "env": self.env,
             "proxy_routes": sorted(list(self.proxy_routes)),
         })
+
+    async def handle_backend(self, request):
+        """Explicit backend testing UI under /backend"""
+        html = self._generate_dev_ui()
+        return web.Response(text=html, content_type="text/html")
     
     async def handle_execute(self, request):
         """Execute the module with test data"""
@@ -559,8 +665,8 @@ class ModuleDevServer:
                 "timestamp": datetime.now().isoformat()
             }
             
-            # Execute module
-            result = self.module.execute(input_data, context)
+            # Execute module via standard run wrapper (validates, handles errors, adds metadata)
+            result = self.module.run(input_data, context)
             
             # Store test result
             self.test_results.append({
@@ -931,7 +1037,7 @@ class ModuleDevServer:
             return
             
         try:
-            # Install dependencies if node_modules doesn't exist
+            # Install dependencies if node_modules doesn't exist, or if required packages are missing
             node_modules = self.frontend_dir / "node_modules"
             if not node_modules.exists():
                 console.print("[blue]Installing frontend dependencies...[/blue]")
@@ -941,18 +1047,80 @@ class ModuleDevServer:
                     check=True,
                     capture_output=True
                 )
+            else:
+                # Verify a few key packages; if missing, run install to recover partial installs
+                required = ["react", "react-dom", "antd", "@ant-design/icons", "recharts", "react-plotly.js"]
+                missing: list[str] = []
+                for pkg in required:
+                    pkg_path = node_modules / Path(*pkg.split("/"))
+                    if not pkg_path.exists():
+                        missing.append(pkg)
+                if missing:
+                    console.print(f"[yellow]Detected missing frontend packages: {', '.join(missing)} — running npm install...[/yellow]")
+                    subprocess.run(
+                        ["npm", "install"],
+                        cwd=self.frontend_dir,
+                        check=True,
+                        capture_output=True
+                    )
             
             # Start webpack dev server
             console.print(f"[blue]Starting frontend dev server on port {self.frontend_port}...[/blue]")
+            popen_kwargs: dict[str, Any] = {
+                "cwd": self.frontend_dir,
+            }
+            if self.verbose:
+                popen_kwargs.update(
+                    stdout=subprocess.PIPE,
+                    stderr=subprocess.STDOUT,
+                    text=True,
+                    bufsize=1,
+                )
+            else:
+                popen_kwargs.update(
+                    stdout=subprocess.DEVNULL,
+                    stderr=subprocess.DEVNULL,
+                )
+
             self.frontend_process = subprocess.Popen(
                 ["npm", "run", "dev"],
-                cwd=self.frontend_dir,
-                stdout=subprocess.PIPE,
-                stderr=subprocess.PIPE
+                **popen_kwargs,
             )
+
+            if self.verbose and self.frontend_process.stdout:
+                self.frontend_log_thread = threading.Thread(
+                    target=self._stream_frontend_output,
+                    args=(self.frontend_process.stdout,),
+                    daemon=True,
+                )
+                self.frontend_log_thread.start()
+
+            # Detect immediate failures (e.g., webpack config errors)
+            time.sleep(0.5)
+            if self.frontend_process.poll() is not None:
+                console.print(
+                    f"[red]Frontend dev server exited early with code {self.frontend_process.returncode}."
+                    + (" Check logs above for details." if self.verbose else " Rerun with --verbose for webpack output.")
+                )
             
         except Exception as e:
             console.print(f"[yellow]Could not start frontend server: {e}[/yellow]")
+
+    def _stream_frontend_output(self, stream):
+        """Forward frontend dev server output to console."""
+        try:
+            for line in iter(stream.readline, ""):
+                if not line:
+                    break
+                console.print(f"[cyan][frontend][/cyan] {line.rstrip()}")
+        except Exception:
+            # Avoid noisy stack traces if the stream is closed unexpectedly
+            pass
+        finally:
+            try:
+                stream.close()
+            except Exception:
+                pass
     
     def _find_available_port(self, start_port: int, max_tries: int = 100) -> int:
         """Find an available port starting from start_port"""
@@ -1020,7 +1188,7 @@ class ModuleDevServer:
         try:
             await site.start()
             console.print(f"\n[green]✓ Dev server running at http://localhost:{self.port}[/green]")
-            console.print("[dim]Open / for backend UI, /ui for frontend (if enabled).[/dim]\n")
+            console.print("[dim]Open /ui for the module UI (UI modules auto-redirect from /). Use /backend for the backend testing UI.[/dim]\n")
             
             # Keep running
             while True:
@@ -1041,6 +1209,14 @@ class ModuleDevServer:
             
             if self.frontend_process:
                 self.frontend_process.terminate()
+                try:
+                    self.frontend_process.wait(timeout=5)
+                except subprocess.TimeoutExpired:
+                    self.frontend_process.kill()
+                self.frontend_process = None
+            if self.frontend_log_thread and self.frontend_log_thread.is_alive():
+                self.frontend_log_thread.join(timeout=1)
+                self.frontend_log_thread = None
             
             if self.http:
                 await self.http.close()
@@ -1060,6 +1236,7 @@ def run_dev_server(
     start_frontend: bool = False,
     frontend_port: int = 3000,
     ca_bundle: str | None = None,
+    verbose: bool = False,
 ):
     """Entry point for dev server"""
     server = ModuleDevServer(
@@ -1073,6 +1250,7 @@ def run_dev_server(
         start_frontend=start_frontend,
         frontend_port=frontend_port,
         ca_bundle=ca_bundle,
+        verbose=verbose,
     )
     
     try:
