@@ -8,9 +8,7 @@ import json
 import shutil
 from typing import Dict, Any, List, Optional
 from pathlib import Path
-from datetime import datetime
-
-from jinja2 import Environment, BaseLoader, Template
+from jinja2 import Environment, BaseLoader
 from string import Template as StrTemplate
 
 
@@ -87,7 +85,7 @@ class CodeGenerator:
             "name": config['name'],
             "version": "1.0.0",
             "type": "with-ui" if config.get('has_ui') else "no-ui",
-            "computeType": "lambda",
+            "computeType": "docker",
             "description": config.get('description', ''),
             "author": config.get('author', {}),
             "runtime": {
@@ -106,6 +104,7 @@ class CodeGenerator:
                 "api": True
             }
         }
+        manifest["entrypoint"] = f"backend.main:{self._to_class_name(config['name'])}"
         
         if config.get('has_ui'):
             manifest['ui'] = {
@@ -128,7 +127,10 @@ Generated by HLA-Compass Module Wizard
 
 from hla_compass import Module
 from typing import Dict, Any, List, Optional
+from datetime import datetime
 import logging
+\n+# Runtime: the platform's container runner (`module-runner`) instantiates this
+# Module directly, so no additional handler function is required.
 {% for dep in imports %}
 import {{ dep }}
 {% endfor %}
@@ -239,12 +241,7 @@ class {{ class_name }}(Module):
         }
     {% endif %}
 
-
-# Lambda handler for AWS Lambda execution
-def lambda_handler(event, context):
-    """AWS Lambda handler"""
-    module = {{ class_name }}()
-    return module.handle_lambda(event, context)
+{{ helper_methods }}
 '''
         
         # Prepare template variables
@@ -261,6 +258,7 @@ def lambda_handler(event, context):
         
         # Generate processing logic based on type
         processing_logic = self._generate_processing_logic(config)
+        helper_methods = self._generate_helper_methods(config)
         
         # Generate output formatting
         output_formatting = self._generate_output_formatting(config)
@@ -283,7 +281,8 @@ def lambda_handler(event, context):
             output_formatting=output_formatting,
             success_return=success_return,
             needs_model='Machine learning' in config.get('processing_type', ''),
-            needs_formatting=True
+            needs_formatting=True,
+            helper_methods=helper_methods
         )
         
         with open(backend_dir / "main.py", "w") as f:
@@ -497,8 +496,9 @@ export default {{ component_name }};
             f.write(code)
     
     def _generate_package_json(self, config: Dict[str, Any], frontend_dir: Path):
-        """Generate package.json for frontend"""
-        package = {
+        """Generate or merge package.json for frontend with Tailwind v4 PostCSS config"""
+        pkg_path = frontend_dir / "package.json"
+        base_pkg = {
             "name": f"{config['name']}-ui",
             "version": "1.0.0",
             "description": f"UI for {config['name']}",
@@ -523,12 +523,46 @@ export default {{ component_name }};
                 "webpack-dev-server": "^4.15.0",
                 "ts-loader": "^9.4.0",
                 "css-loader": "^6.8.0",
-                "style-loader": "^3.3.0"
+                "style-loader": "^3.3.0",
+                "tailwindcss": "^4.1.13",
+                "@tailwindcss/postcss": "^4.1.13",
+                "autoprefixer": "^10.4.20",
+                "postcss": "^8.5.1",
+                "postcss-loader": "^8.1.0"
             }
         }
-        
-        with open(frontend_dir / "package.json", "w") as f:
-            json.dump(package, f, indent=2)
+
+        if pkg_path.exists():
+            try:
+                with open(pkg_path, "r") as f:
+                    existing = json.load(f)
+            except Exception:
+                existing = {}
+            # Merge: preserve existing dependencies/devDependencies, override name/description/scripts minimally
+            pkg = existing or {}
+            pkg.setdefault("name", base_pkg["name"])  # don't force rename if user edited
+            pkg.setdefault("version", base_pkg["version"])
+            pkg.setdefault("description", base_pkg["description"])
+            scripts = pkg.get("scripts", {})
+            scripts.setdefault("build", base_pkg["scripts"]["build"])
+            scripts.setdefault("dev", base_pkg["scripts"]["dev"])
+            scripts.setdefault("test", base_pkg["scripts"]["test"])
+            pkg["scripts"] = scripts
+            deps = pkg.get("dependencies", {})
+            deps.setdefault("react", base_pkg["dependencies"]["react"])
+            deps.setdefault("react-dom", base_pkg["dependencies"]["react-dom"])
+            deps.setdefault("antd", base_pkg["dependencies"]["antd"])
+            deps.setdefault("@ant-design/icons", base_pkg["dependencies"]["@ant-design/icons"])
+            pkg["dependencies"] = deps
+            dev_deps = pkg.get("devDependencies", {})
+            for k, v in base_pkg["devDependencies"].items():
+                dev_deps.setdefault(k, v)
+            pkg["devDependencies"] = dev_deps
+        else:
+            pkg = base_pkg
+
+        with open(pkg_path, "w") as f:
+            json.dump(pkg, f, indent=2)
     
     def _generate_readme(self, config: Dict[str, Any], output_dir: Path):
         """Generate README.md"""
@@ -791,14 +825,24 @@ Generated by HLA-Compass Module Wizard
         if 'Sequence analysis' in proc_type:
             return """
         # Example: Analyze peptide sequences
+        if not peptide_sequences:
+            return []
+
         results = []
         for seq in peptide_sequences:
+            if not seq:
+                continue
+
+            sequence = str(seq).strip()
+            if not sequence:
+                continue
+
             # Calculate properties
-            length = len(seq)
-            hydrophobicity = self._calculate_hydrophobicity(seq)
+            length = len(sequence)
+            hydrophobicity = self._calculate_hydrophobicity(sequence)
             
             results.append({
-                'sequence': seq,
+                'sequence': sequence,
                 'length': length,
                 'hydrophobicity': hydrophobicity
             })
@@ -826,6 +870,57 @@ Generated by HLA-Compass Module Wizard
             }
         ]
         """
+
+    def _generate_helper_methods(self, config: Dict[str, Any]) -> str:
+        """Generate additional helper methods based on processing type."""
+        proc_type = config.get('processing_type', '')
+
+        if 'Sequence analysis' in proc_type:
+            return """
+    def _calculate_hydrophobicity(self, sequence: str) -> float:
+        \"\"\"Calculate average hydrophobicity using the Kyte-Doolittle scale.\"\"\"
+        if not sequence:
+            return 0.0
+
+        scale = {
+            'A': 1.8,
+            'R': -4.5,
+            'N': -3.5,
+            'D': -3.5,
+            'C': 2.5,
+            'Q': -3.5,
+            'E': -3.5,
+            'G': -0.4,
+            'H': -3.2,
+            'I': 4.5,
+            'L': 3.8,
+            'K': -3.9,
+            'M': 1.9,
+            'F': 2.8,
+            'P': -1.6,
+            'S': -0.8,
+            'T': -0.7,
+            'W': -0.9,
+            'Y': -1.3,
+            'V': 4.2,
+        }
+
+        total = 0.0
+        counted = 0
+        for residue in sequence.upper():
+            value = scale.get(residue)
+            if value is None:
+                continue
+            total += value
+            counted += 1
+
+        if counted == 0:
+            return 0.0
+
+        return total / counted
+"""
+
+        return ""
     
     def _generate_output_formatting(self, config: Dict[str, Any]) -> str:
         """Generate output formatting code"""
@@ -947,4 +1042,4 @@ Generated by HLA-Compass Module Wizard
                 line += f": {spec['description']}"
             lines.append(line)
         
-        return '\n'.join(lines)
\ No newline at end of file
+        return '\n'.join(lines)
