@@ -8,7 +8,7 @@
  * with Tailwind CSS classes and scientific styling patterns.
  */
 
-import React, { useEffect, useMemo, useState, useCallback } from 'react';
+import React, { useEffect, useMemo, useState, useCallback, lazy, Suspense } from 'react';
 import './styles.css';
 import { 
   Button,
@@ -43,12 +43,12 @@ import {
 
 import { apiGet, devPost, devGet } from './api';
 import LocalDataBrowser from './LocalDataBrowser';
-import ChartDemo from './ChartDemo';
 import { SearchOutlined, ClearOutlined, ExperimentOutlined } from '@ant-design/icons';
 import CopyButton from './CopyButton';
 import { applyTheme, createAntdTheme } from './theme';
 
 const { Title, Text, Paragraph } = Typography;
+const ChartDemo = lazy(() => import('./ChartDemo'));
 
 // Module props interface
 interface ModuleProps {
@@ -92,6 +92,32 @@ const ModuleUI: React.FC<ModuleProps> = ({ onExecute, initialParams }) => {
     }
   }, []);
 
+  // Load Inter font to match platform design
+  useEffect(() => {
+    const fontUrl = "https://fonts.googleapis.com/css2?family=Inter:wght@100..900&display=swap";
+    // Check if font is already loaded
+    if (document.querySelector(`link[href="${fontUrl}"]`)) {
+      return;
+    }
+
+    const preconnect1 = document.createElement('link');
+    preconnect1.rel = 'preconnect';
+    preconnect1.href = 'https://fonts.googleapis.com';
+
+    const preconnect2 = document.createElement('link');
+    preconnect2.rel = 'preconnect';
+    preconnect2.href = 'https://fonts.gstatic.com';
+    preconnect2.setAttribute('crossorigin', '');
+
+    const fontLink = document.createElement('link');
+    fontLink.rel = 'stylesheet';
+    fontLink.href = fontUrl;
+
+    document.head.appendChild(preconnect1);
+    document.head.appendChild(preconnect2);
+    document.head.appendChild(fontLink);
+  }, []);
+
   const antdThemeConfig = useMemo(() => {
     if (hostHasAntVars) return undefined; // inherit host platform theme
     return createAntdTheme('system');
@@ -115,8 +141,10 @@ const ModuleUI: React.FC<ModuleProps> = ({ onExecute, initialParams }) => {
         // Add more parameters as needed
       };
 
-      // Execute the module via local dev server
-      const result = await devPost('/execute', { input: params });
+      // Execute the module via provided host callback when embedded, otherwise via local dev server
+      const result = onExecute
+        ? await onExecute(params)
+        : await devPost('/execute', { input: params });
 
       // Handle the response
       if (result.status === 'success') {
@@ -229,7 +257,8 @@ const ModuleUI: React.FC<ModuleProps> = ({ onExecute, initialParams }) => {
       setApiDemoOutput(samples);
       message.success('apiGet samples completed');
     } catch (e: any) {
-      setApiDemoError(e.message || 'apiGet failed');
+      const tip = " — Tip: enable online mode with 'hla-compass dev --online' and login via 'hla-compass auth login'";
+      setApiDemoError((e.message || 'apiGet failed') + tip);
       setApiDemoOutput(null);
     }
   };
@@ -362,6 +391,29 @@ const ModuleUI: React.FC<ModuleProps> = ({ onExecute, initialParams }) => {
       {loading && (
         <div className="text-center py-6"><Spin /></div>
       )}
+
+      {(results || summary) && (
+        <>
+          <Divider />
+          <Title level={4} className="text-gray-800 mb-2">Results</Title>
+          {summary && (
+            <Card size="small" className="mb-3">
+              <pre className="bg-white p-3 rounded border text-sm font-mono overflow-auto max-h-60">{JSON.stringify(summary, null, 2)}</pre>
+            </Card>
+          )}
+          {Array.isArray(results) ? (
+            <Table
+              dataSource={results as any}
+              columns={columns}
+              rowKey="id"
+              className="scientific-table"
+              pagination={{ pageSize: 10, showSizeChanger: true }}
+            />
+          ) : results ? (
+            <pre className="bg-white p-3 rounded border text-sm font-mono overflow-auto max-h-80">{JSON.stringify(results, null, 2)}</pre>
+          ) : null}
+        </>
+      )}
     </Card>
   );
 
@@ -399,6 +451,9 @@ const ModuleUI: React.FC<ModuleProps> = ({ onExecute, initialParams }) => {
       <div className="mb-4">
         <Title level={4} className="text-gray-800 mb-2">API Demos</Title>
         <Text className="text-gray-600">Demonstrates devPost, devGet, and apiGet usage.</Text>
+        <div className="mt-2">
+          <Alert type="info" showIcon message="Tip: To call real API endpoints, start dev with --online and login via 'hla-compass auth login'. You can also specify --ca-bundle for custom trust stores." />
+        </div>
       </div>
       <Space direction="vertical" style={{ width: '100%' }} size="large">
         <div>
@@ -462,6 +517,18 @@ const ModuleUI: React.FC<ModuleProps> = ({ onExecute, initialParams }) => {
     </div>
   );
 
+  const Charts = (
+    <Suspense
+      fallback={
+        <div className="flex justify-center py-10">
+          <Spin size="large" tip="Loading charts..." />
+        </div>
+      }
+    >
+      <ChartDemo />
+    </Suspense>
+  );
+
   const content = (
     <div className="module-container max-w-screen-lg mx-auto p-5 space-y-5">
       <Tabs
@@ -472,7 +539,7 @@ const ModuleUI: React.FC<ModuleProps> = ({ onExecute, initialParams }) => {
           { key: 'forms', label: 'Forms & Validation', children: FormsAndValidation },
           { key: 'tables', label: 'Tables & Data', children: TablesAndData },
           { key: 'api', label: 'API Demos', children: ApiDemos },
-          { key: 'charts', label: 'Charts', children: <ChartDemo /> },
+          { key: 'charts', label: 'Charts', children: Charts },
           { key: 'theming', label: 'Theming', children: Theming },
           { key: 'browser', label: 'Local Data Browser', children: DataBrowser },
         ]}
@@ -490,4 +557,11 @@ const ModuleUI: React.FC<ModuleProps> = ({ onExecute, initialParams }) => {
   );
 };
 
+// Expose the component on window for the dev server UMD wrapper fallback
+// This ensures /ui can mount the UI even if the bundler doesn't emit the UMD global as expected.
+if (typeof window !== 'undefined') {
+  // @ts-ignore
+  (window as any).ModuleUI = ModuleUI;
+}
+
 export default ModuleUI;
