@@ -9,7 +9,12 @@ module.exports = {
     rules: [
       {
         test: /\.tsx?$/,
-        use: 'ts-loader',
+        use: {
+          loader: 'ts-loader',
+          options: {
+            transpileOnly: true,
+          },
+        },
         exclude: /node_modules/,
       },
       {
@@ -17,17 +22,7 @@ module.exports = {
         use: [
           'style-loader',
           'css-loader',
-          {
-            loader: 'postcss-loader',
-            options: {
-              postcssOptions: {
-                plugins: [
-                  require('tailwindcss'),
-                  require('autoprefixer'),
-                ],
-              },
-            },
-          },
+          'postcss-loader',
         ],
       },
     ],
@@ -35,14 +30,21 @@ module.exports = {
   output: {
     filename: 'bundle.js',
     path: path.resolve(__dirname, 'dist'),
-    library: 'ModuleUI',
-    libraryTarget: 'umd',
+    library: {
+      name: 'ModuleUI',
+      type: 'umd',
+      export: 'default',
+    },
+    globalObject: "typeof self !== 'undefined' ? self : this",
+    clean: true,
   },
   externals: {
     react: 'React',
     'react-dom': 'ReactDOM',
     antd: 'antd',
     '@ant-design/icons': 'icons',
+    'plotly.js': 'Plotly',
+    'plotly.js-dist-min': 'Plotly',
   },
   devServer: {
     static: {
@@ -51,5 +53,11 @@ module.exports = {
     compress: true,
     port: 3000,
     hot: true,
+    headers: {
+      'Access-Control-Allow-Origin': '*',
+    },
+    devMiddleware: {
+      writeToDisk: true,
+    },
   },
 };
