@@ -6,9 +6,10 @@ import logging
 import importlib.util
 from pathlib import Path
 from typing import Any, Callable
-from datetime import datetime
+from datetime import datetime, UTC
 import sys
 import os
+import uuid
 
 from .types import ExecutionContext
 from .module import Module
@@ -17,10 +18,20 @@ from .module import Module
 logger = logging.getLogger(__name__)
 
 
+def _now_utc() -> datetime:
+    return datetime.now(UTC)
+
+
 class MockAPI:
     """Mock API client for testing"""
 
-    def __init__(self, data: dict[str, Any] | None = None, use_devkit: bool = False):
+    def __init__(
+        self,
+        data: dict[str, Any] | None = None,
+        *,
+        use_devkit: bool = False,
+        use_real_storage: bool = False,
+    ):
         """
         Initialize mock API with test data.
 
@@ -31,8 +42,9 @@ class MockAPI:
         self.data = data or self._get_default_data()
         self.call_history = []
         self.use_devkit = use_devkit
+        self.use_real_storage = use_real_storage
 
-        if use_devkit:
+        if self.use_devkit:
             self._setup_devkit_connection()
 
     def _setup_devkit_connection(self):
@@ -99,7 +111,7 @@ class MockAPI:
     def _record_call(self, method: str, **kwargs):
         """Record API call for verification"""
         self.call_history.append(
-            {"method": method, "kwargs": kwargs, "timestamp": datetime.utcnow()}
+            {"method": method, "kwargs": kwargs, "timestamp": _now_utc()}
         )
 
     def get_peptides(
@@ -175,7 +187,7 @@ class MockStorage:
             "body": body,
             "content_type": content_type,
             "metadata": metadata or {},
-            "timestamp": datetime.utcnow(),
+            "timestamp": _now_utc(),
         }
 
         return f"s3://test-bucket/{key}"
@@ -205,15 +217,10 @@ class MockContext:
     """Create mock execution context for testing"""
 
     @staticmethod
-    def _create_storage_client(use_real_storage: bool = False):
+    def _create_storage_client(*, opt_in: bool = False):
         """Create storage client for testing"""
-        # Check if we're in local development
-        is_local = (
-            os.getenv("ENVIRONMENT") == "local" or os.getenv("HLA_ENV") == "local"
-        )
-
-        # Use real storage if requested or in local development with MinIO
-        if use_real_storage or (is_local and os.getenv("S3_ENDPOINT_URL")):
+        # Only use real storage when explicitly requested
+        if opt_in:
             try:
                 import boto3
                 from botocore.client import Config
@@ -251,7 +258,9 @@ class MockContext:
         organization_id: str = "test-org",
         tier: str = "foundational",
         api_data: dict[str, Any] | None = None,
+        use_devkit: bool = False,
         use_real_storage: bool = False,
+        mode: str = "interactive",
     ) -> ExecutionContext:
         """
         Create mock execution context.
@@ -262,7 +271,9 @@ class MockContext:
             organization_id: Organization identifier
             tier: Subscription tier
             api_data: Optional API test data
+            use_devkit: Whether to connect to local devkit services
             use_real_storage: Use real S3/MinIO instead of mock
+            mode: Execution mode flag ("interactive", "async", or "workflow") propagated to module context
 
         Returns:
             Mock execution context
@@ -272,9 +283,14 @@ class MockContext:
             "user_id": user_id,
             "organization_id": organization_id,
             "tier": tier,
-            "api": MockAPI(api_data),
-            "storage": MockContext._create_storage_client(use_real_storage),
-            "execution_time": datetime.utcnow(),
+            "mode": mode,
+            "api": MockAPI(
+                api_data,
+                use_devkit=use_devkit,
+                use_real_storage=use_real_storage,
+            ),
+            "storage": MockContext._create_storage_client(opt_in=use_real_storage),
+            "execution_time": _now_utc(),
         }
 
 
@@ -285,10 +301,16 @@ class ModuleTester:
         """Initialize module tester"""
         self.logger = logging.getLogger(f"{__name__}.ModuleTester")
         self.use_devkit = False
+        self.use_real_storage = False
 
-    def configure_local_devkit(self):
-        """Configure tester to use local devkit container"""
+    def configure_local_devkit(self, *, use_real_storage: bool = False):
+        """Configure tester to use local devkit container.
+
+        Args:
+            use_real_storage: When True, use the configured S3/MinIO client instead of the in-memory mock.
+        """
         self.use_devkit = True
+        self.use_real_storage = use_real_storage
 
     def test_local(
         self,
@@ -308,7 +330,10 @@ class ModuleTester:
             Module execution result
         """
         if context is None:
-            context = MockContext.create()
+            context = MockContext.create(
+                use_devkit=self.use_devkit,
+                use_real_storage=self.use_real_storage,
+            )
 
         # Load module
         module = self._load_module(module_path)
@@ -330,19 +355,51 @@ class ModuleTester:
             raise ValueError(f"Module not found: {module_path}")
 
         # Load module dynamically
-        spec = importlib.util.spec_from_file_location("test_module", module_path)
+        module_name = f"hla_compass_test_{uuid.uuid4().hex}"
+        spec = importlib.util.spec_from_file_location(module_name, module_path)
         if spec is None or spec.loader is None:
             raise ValueError(f"Cannot load module from {module_path}")
 
         module = importlib.util.module_from_spec(spec)
-        sys.modules["test_module"] = module
-        spec.loader.exec_module(module)
+        previous = sys.modules.pop(module_name, None)
+        sys.modules[module_name] = module
+        try:
+            spec.loader.exec_module(module)
 
-        # Get execute function
-        if not hasattr(module, "execute"):
-            raise ValueError("Module must have an 'execute' function")
+            # Prefer top-level execute for simple function modules
+            if hasattr(module, "execute"):
+                return module.execute
 
-        return module.execute
+            # Otherwise, find a Module subclass and wrap it to call run()
+            mod_class = None
+            for name in dir(module):
+                obj = getattr(module, name)
+                try:
+                    from .module import Module as _BaseModule  # local import to avoid cycles
+                except Exception:
+                    _BaseModule = Module
+                if (
+                    isinstance(obj, type)
+                    and issubclass(obj, _BaseModule)
+                    and obj is not _BaseModule
+                ):
+                    mod_class = obj
+                    break
+
+            if not mod_class:
+                raise ValueError(
+                    "Module must have an 'execute' function or a class inheriting from hla_compass.Module"
+                )
+
+            def _runner(input_data, context):
+                instance = mod_class()
+                return instance.run(input_data, context)
+
+            return _runner
+        finally:
+            sys.modules.pop(module_name, None)
+            if previous is not None:
+                sys.modules[module_name] = previous
 
     def test_with_class(
         self,
@@ -362,7 +419,10 @@ class ModuleTester:
             Module execution result
         """
         if context is None:
-            context = MockContext.create()
+            context = MockContext.create(
+                use_devkit=self.use_devkit,
+                use_real_storage=self.use_real_storage,
+            )
 
         # Create module instance
         module = module_class()
@@ -370,6 +430,65 @@ class ModuleTester:
         # Execute using run method
         return module.run(input_data, context)
 
+    def quickstart(
+        self,
+        module_class: type[Module],
+        sample_input: dict[str, Any] | None = None,
+        *,
+        mode: str = "interactive",
+        use_devkit: bool | None = None,
+        use_real_storage: bool | None = None,
+    ) -> dict[str, Any]:
+        """
+        Run a module class using a generated context and sample data.
+
+        Args:
+            module_class: Module subclass to execute.
+            sample_input: Optional input payload (defaults to manifest defaults).
+            mode: Execution mode to stamp into the context ("interactive", "async", "workflow").
+            use_devkit: Override tester devkit flag for this run.
+            use_real_storage: Override tester storage flag for this run.
+
+        Returns:
+            Module execution result dictionary.
+        """
+        manifest_defaults = self._derive_defaults_from_manifest(module_class)
+        input_payload = sample_input or manifest_defaults or {}
+
+        context = MockContext.create(
+            use_devkit=self.use_devkit if use_devkit is None else use_devkit,
+            use_real_storage=self.use_real_storage if use_real_storage is None else use_real_storage,
+            mode=mode,
+        )
+
+        self.logger.info("Running module quickstart with payload: %s", input_payload)
+        return self.test_with_class(module_class, input_payload, context)
+
+    def _derive_defaults_from_manifest(self, module_class: type[Module]) -> dict[str, Any]:
+        try:
+            module_instance = module_class()
+        except Exception as exc:
+            self.logger.warning("Could not instantiate module class %s: %s", module_class.__name__, exc)
+            return {}
+
+        inputs = module_instance.manifest.get("inputs", {})
+        defaults: dict[str, Any] = {}
+
+        if isinstance(inputs, dict):
+            if inputs.get("type") == "object" and "properties" in inputs:
+                for name, schema in inputs.get("properties", {}).items():
+                    if "default" in schema:
+                        defaults[name] = schema["default"]
+            else:
+                for name, schema in inputs.items():
+                    if isinstance(schema, dict) and "default" in schema:
+                        defaults[name] = schema["default"]
+
+        if not defaults:
+            self.logger.info("No defaults found in manifest; quickstart will use empty input payload")
+
+        return defaults
+
     def create_test_suite(
         self, module_path: str, test_cases: list[dict[str, Any]]
     ) -> dict[str, Any]:
