@@ -4,8 +4,6 @@ Interactive wizard for HLA-Compass module creation
 Guides users through module setup with intelligent questions and code generation.
 """
 
-import json
-import os
 import re
 import shutil
 import subprocess
